	/*
 * Copyright (C) 2005 Cesar Kallas
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * E-mail: cesarkallas@gmx.net
 * http://cesarkallas.soulivre.org
 *
 */

package estella.cliente;
import  estella.util.Log;
import  estella.util.Constantes;

import  java.net.Socket;
import  java.net.ServerSocket;
import  java.io.IOException;
import  java.io.DataInputStream;
import  java.io.DataOutputStream;
import  java.io.BufferedReader;
import  java.io.InputStreamReader;
import  java.net.SocketException;
import  java.net.SocketTimeoutException;
import  java.lang.String;

/**
* Funcoes para gerenciar conexoes de saida
* @author Cesar Kallas - cesarkallas@gmx.net
* @since 22/11/2005
*/
public class Conexao
{
  private Log LOG;
  private Constantes CON;
  private Socket socket;
	private DataInputStream receive;
	private DataOutputStream send;
  private BufferedReader bufferSocket;
  private String host;
  private int porta;

  /**
  * Instancia uma nova conexao
  * @param host String com o host no formato host.com ou 127.0.0.1
  * @param porta Inteiro contendo um numero de porta
  */
  public Conexao(String host, int porta)
  {
    this.host = host;
    this.porta = porta;
  }
  public Conexao(Socket sckt)
  {
    this.socket = sckt;
  }

  /**
  * Cria um socket cliente, setando os parametros necessarios
  * @return true ou false / se a conexao foi criada e se modo leitura e gravacao estao prontos
  */  
  public boolean inicia()
	{
    CON = new Constantes();
    LOG = new Log();
    
    // Cria um novo socket, se ele nao foi passado na criacao da classe
    if(socket == null)
      try {
        socket = new Socket(host, porta);
      }
      catch(IOException FalhaConectar) {
        LOG.falha(FalhaConectar.getMessage() + " @ " + host + ":" + porta);
        return false;
      }
      catch(SecurityException Firewall) {
        LOG.atencao(Firewall.getMessage()  + " @ " + host + ":" + porta);
        return false;
      }
      catch(Exception FalhaGeral) {
        LOG.falha(FalhaGeral.getMessage()  + " @ " + host + ":" + porta);
        return false;
      }
      
    // Inicia leitura e gravacao no socket
    try {
      receive = new DataInputStream(socket.getInputStream());
      send = new DataOutputStream(socket.getOutputStream());
      LOG.info("Socket cliente (leitura/gravacao): " + socket);
		}
    catch(IOException FalhaCanal) {
      LOG.falha(FalhaCanal.getMessage());
      return false;
    }
    
    try {
      socket.setSoTimeout(CON.SOCKET_TIMEOUT);
      socket.setTcpNoDelay(CON.SOCKET_TCPNODELAY);
      return true;
    }
    catch (SocketException socketTimeout) {
      //Log.falha(socketTimeout.getMessage());
      throw(new RuntimeException(socketTimeout));
    }
  }
  
  /**
  * Escreve dados no socket
  * @param msg - String com a mensagem a ser escrita no socket
  * @return true ou false
  */
  public boolean envia(String msg)
  {
    try {
      msg = msg + "\n";
      send.write(msg.getBytes());
      send.flush();
      LOG.info(msg + " --> " + socket);
      return true;
    }
    catch(Exception EnviarMsg) {
      LOG.falha(EnviarMsg.getMessage() + socket);
      return false;
    }
  }
  
  /**
  * Recebe dados do socket (com timeout, vide classe constantes)
  * @return msg - String com a mensagem recebida ou null - Problema na recepcao de dados
  */
  public String recebe()
  {
    String msg = new String();
    try {
      bufferSocket = new BufferedReader(new InputStreamReader(receive));
      
      try {
        msg = bufferSocket.readLine();
      }
      catch (SocketTimeoutException socketTimeout) {
        envia(CON.ESTELLA + "\r\n" + CON.CONNECTION_TIMEOUT);
        fecha();
        return null;
      }
      
      LOG.info(socket + " <-- " + msg);
      return(msg);
    }
    catch(Exception ReceberMsg) {
      LOG.falha(ReceberMsg.getMessage() + socket);
      return null;
    }
  }
  
  /**
  * Fecha conexao cliente, incluindo os canais de comunicacao (leitura / escrita)
  * @return true ou false
  */
  public boolean fecha()
  {
    String sck_info;
    try {
      sck_info = "" + socket;
      receive.close();
      send.close();
      socket.close();
      LOG.info("Cliente desconectado: " + sck_info);
      return true;
    }
    catch(IOException FalhaFecharSocket) {
      LOG.falha(FalhaFecharSocket.getMessage());
      return false;
    }
  }
}
