/*
 * Copyright (C) 2005 Cesar Kallas
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * E-mail: cesarkallas@gmx.net
 * http://cesarkallas.soulivre.org
 *
 */

/**
* Funcoes do servidor
* @author Cesar Kallas - cesarkallas@gmx.net
* @since 23/11/2005
*/
package estella.servidor;
import  estella.util.Log;
import  estella.servidor.ConexaoServidor;
import  estella.cliente.Conexao;

import  java.io.IOException;
import  java.lang.Thread;
import  java.net.Socket;
import  java.net.SocketTimeoutException;

public class Servidor
{
  private ConexaoServidor conexaoServidor;
  private Conexao conexao;
  private Socket sck_cliente;
  private Thread t_conexao;
  private Log LOG;

  public Servidor()
  {
    conexaoServidor = new ConexaoServidor();
    LOG = new Log();
    inicia();
  }
  
  /**
    * Inicia um novo servico de escuta em uma porta
    */
  public void inicia()
  {
    conexaoServidor.inicia();
    
    // Escuta eternamente requisicao no servidor
    while(true) {
      try {
      sck_cliente = conexaoServidor.escuta();
      System.out.println("\nServidor recebeu nova conexao\n");
      }
      catch(Exception FalhaEscutarServidor)
      {
        LOG.falha(FalhaEscutarServidor.getMessage());
        return;
      }
      
      // Nova requisicao, ela eh null ? (pode ser quando o cliente conecta e desconecta)
      // Se ok, cria uma nova thread para tratar a requisicao
      if(sck_cliente != null) 
        try {
          System.out.println("\nServidor criando nova thread para requisicao\n");
          t_conexao = new Thread(new trataRequisicao(sck_cliente));
          t_conexao.start();
        }
        catch(Exception threadConexao) {
          LOG.falha(threadConexao.getMessage());
          return;
        }
    }
  }
}

/**
* Thread para tratar requisicao recebida pelo servidor
* @param sck_cliente Socket com a conexao do cliente
* @since 23/11/2005
* @author Cesar Kallas - cesarkallas at gmx . net
*/
class trataRequisicao implements Runnable
{
  private Socket sck_cliente;
  private Conexao conexao;

  public trataRequisicao(Socket sck_cliente)
  {
    conexao = new Conexao(sck_cliente);
    conexao.inicia();
  }

  public void run()
  {
	String msg = new String(conexao.recebe());
	  
    while(msg != null) {
		System.out.println("Nova mensagem: " + msg);
		msg = conexao.recebe();
	}
  }
}
